/*

AWD file loading example in Away3d

Demonstrates:

How to use the Loader3D object to load an embedded internal awd model.

Code by Rob Bateman
rob@infiniteturtles.co.uk
http://www.infiniteturtles.co.uk

This code is distributed under the MIT License

Copyright (c) The Away Foundation http://www.theawayfoundation.org

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the “Software”), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

 */
package
{
    import flash.display.Sprite;
    import flash.display.StageAlign;
    import flash.display.StageScaleMode;
    import flash.events.Event;
    import flash.events.KeyboardEvent;
    import flash.events.MouseEvent;
    import flash.filters.DropShadowFilter;
    import flash.geom.Vector3D;
    import flash.net.URLRequest;
    import flash.text.AntiAliasType;
    import flash.text.GridFitType;
    import flash.text.TextField;
    import flash.text.TextFormat;
    import flash.ui.Keyboard;
    
    import away3d.animators.SkeletonAnimator;
    import away3d.animators.nodes.SkeletonClipNode;
    import away3d.animators.transitions.CrossfadeTransition;
    import away3d.cameras.lenses.PerspectiveLens;
    import away3d.containers.View3D;
    import away3d.controllers.HoverController;
    import away3d.debug.AwayStats;
    import away3d.events.AnimationStateEvent;
    import away3d.events.AssetEvent;
    import away3d.library.AssetLibrary;
    import away3d.library.assets.AssetType;
    import away3d.loaders.Loader3D;
    import away3d.loaders.parsers.AWD2Parser;
	
	
    [SWF(backgroundColor="#333338", frameRate="60", quality="LOW")]
    public class Intermediate_AWDViewer extends Sprite
	{
    	//signature swf
    	[Embed(source="/../embeds/signature.swf", symbol="Signature")]
    	public var SignatureSwf:Class;
		
        //engine variables
        private var _view:View3D;
		private var _signature:Sprite;
        private var _stats:AwayStats;
        private var _cameraController:HoverController;
        private var _animator:SkeletonAnimator;
        
        //navigation
        private var _prevMouseX:Number;
        private var _prevMouseY:Number;
        private var _mouseMove:Boolean;
        private var _cameraHeight:Number = 0;
		private var _stateTransition:CrossfadeTransition = new CrossfadeTransition(0.5);
		private const IDLE_NAME:String = "idle02";
		
        private var _text:TextField;
        
        /**
         * Constructor
         */
        public function Intermediate_AWDViewer()
		{
            init();
        }
        
        /**
         * Global initialise function
         */
        private function init():void
		{
            initEngine();
            initText();
			initListeners();
			
			AssetLibrary.enableParser(AWD2Parser);
			
			//kickoff asset loading
			var loader:Loader3D = new Loader3D();
			loader.load(new URLRequest("assets/shambler.awd"));
			loader.addEventListener(AssetEvent.ASSET_COMPLETE, onAssetComplete);
			
			_view.scene.addChild(loader);
        }
        
        /**
         * Initialise the engine
         */
        private function initEngine():void
		{
            stage.scaleMode = StageScaleMode.NO_SCALE;
            stage.align = StageAlign.TOP_LEFT;
            
			//create the view
            _view = new View3D();
			_view.backgroundColor = 0x333338;
            addChild(_view);
            
			//create custom lens
            _view.camera.lens = new PerspectiveLens(70);
            _view.camera.lens.far = 5000;
            _view.camera.lens.near = 1;
            
			//setup controller to be used on the camera
            _cameraController = new HoverController(_view.camera, null, 0, 0, 150, 10, 90);
			_cameraController.lookAtPosition = new Vector3D(0,60, 0);
            _cameraController.tiltAngle = 0;
            _cameraController.panAngle = 0;
            _cameraController.minTiltAngle = 5;
            _cameraController.maxTiltAngle = 60;
            _cameraController.autoUpdate = false;
            
            
            //add signature
			addChild(_signature = new SignatureSwf());
            
            //add stats
            addChild(_stats = new AwayStats(_view, true, true));
        }
		
		/**
         * Create an instructions overlay
         */
        private function initText():void
		{
            _text = new TextField();
            _text.defaultTextFormat = new TextFormat("Verdana", 11, 0xFFFFFF);
			_text.embedFonts = true;
			_text.antiAliasType = AntiAliasType.ADVANCED;
			_text.gridFitType = GridFitType.PIXEL;
            _text.width = 300;
            _text.height = 250;
            _text.selectable = false;
            _text.mouseEnabled = true;
            _text.wordWrap = true;
            _text.filters = [new DropShadowFilter(1, 45, 0x0, 1, 0, 0)];
            addChild(_text);
        }
        
        /**
         * Initialise the listeners
         */
        private function initListeners():void
		{
            //add render loop
            addEventListener(Event.ENTER_FRAME, onEnterFrame);
			
            //navigation
            stage.addEventListener(MouseEvent.MOUSE_DOWN, onStageMouseDown);
            stage.addEventListener(MouseEvent.MOUSE_MOVE, onStageMouseMove);
            stage.addEventListener(MouseEvent.MOUSE_UP, onStageMouseLeave);
            stage.addEventListener(MouseEvent.MOUSE_WHEEL, onStageMouseWheel);
            stage.addEventListener(Event.MOUSE_LEAVE, onStageMouseLeave);
			stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyDown);
			
            //add resize event
            stage.addEventListener(Event.RESIZE, onResize);
            onResize();
        }
		
		/**
		 * loader listener for asset complete events
		 */		
		private function onAssetComplete(event:AssetEvent):void
		{
			trace(event.asset.name);
			if (event.asset.assetType == AssetType.ANIMATOR) {
				_animator = event.asset as SkeletonAnimator;
				_animator.play(IDLE_NAME);
			} else if (event.asset.assetType == AssetType.ANIMATION_NODE) {
				
				var node:SkeletonClipNode = event.asset as SkeletonClipNode;
				
				if (node.name == IDLE_NAME) {
					node.looping = true;
				} else {
					node.looping = false;
					node.addEventListener(AnimationStateEvent.PLAYBACK_COMPLETE, onPlaybackComplete);
				}
			}
		}
		
		/**
		 * Key down listener for animation
		 */
		private function onKeyDown(event:KeyboardEvent):void
		{
			switch (event.keyCode) {
				case Keyboard.NUMBER_1:
					playAction("attack01");
					break;
				case Keyboard.NUMBER_2:
					playAction("attack02");
					break;
				case Keyboard.NUMBER_3:
					playAction("attack03");
					break;
				case Keyboard.NUMBER_4:
					playAction("attack04");
					break;
				case Keyboard.NUMBER_5:
					playAction("attack05");
			}
		}
		
		private function playAction(name:String):void
		{
			_animator.play(name, _stateTransition, 0);
		}
		
		private function onPlaybackComplete(event:AnimationStateEvent):void
		{
			if (_animator.activeState != event.animationState)
				return;
			
			_animator.play(IDLE_NAME, _stateTransition);
		}
		
        /**
         * Render loop
         */
        private function onEnterFrame(event:Event):void
		{
			
            //update camera controler
            _cameraController.update();
		
			
            //update view
            _view.render();
        }
    
		
        /**
         * stage listener and mouse control
         */
        private function onResize(event:Event=null):void
		{
            _view.width = stage.stageWidth;
            _view.height = stage.stageHeight;
            _stats.x = stage.stageWidth - _stats.width;
            _signature.y = stage.stageHeight - _signature.height;
        }
        
        private function onStageMouseDown(ev:MouseEvent):void
		{
            _prevMouseX = ev.stageX;
            _prevMouseY = ev.stageY;
            _mouseMove = true;
        }
        
        private function onStageMouseLeave(event:Event):void
		{
            _mouseMove = false;
        }
        
        private function onStageMouseMove(ev:MouseEvent):void
		{
            if (_mouseMove) {
                _cameraController.panAngle += (ev.stageX - _prevMouseX);
                _cameraController.tiltAngle += (ev.stageY - _prevMouseY);
            }
            _prevMouseX = ev.stageX;
            _prevMouseY = ev.stageY;
        }
        
        /**
         * mouseWheel listener
         */
        private function onStageMouseWheel(ev:MouseEvent):void
		{
            _cameraController.distance -= ev.delta * 5;
			
			_cameraHeight = (_cameraController.distance < 600)? (600 - _cameraController.distance)/2 : 0;
			
            if (_cameraController.distance < 100)
                _cameraController.distance = 100;
            else if (_cameraController.distance > 2000)
                _cameraController.distance = 2000;
        }
    }
}
